;Nach Tutorial auf Roguelike Basein
;http://roguebasin.roguelikedevelopment.org/index.php/Complete_Roguelike_Tutorial,_using_python%2Blibtcod,_part_1

CompilerIf #PB_Compiler_Unicode
    CompilerError "unicode ausschalten"
CompilerEndIf
  
CompilerIf #PB_Compiler_Debugger=0
    CompilerError "debugger anschalten"
CompilerEndIf

IncludeFile "libtcod\libtcod.pbi"

;- ++ Define ++

;- -- Const
; - Screen
#SCREEN_WIDTH = 80
#SCREEN_HEIGHT = 50

; - Map
#MAP_WIDTH = 80
#MAP_HEIGHT = 45

#ROOM_MAX_SIZE = 15
#ROOM_MINSIZE = 6
#MAX_ROOMS = 30

; - FOV
#FOV_ALGO = #FOV_BASIC  ;default FOV algorithm
#FOV_LIGHT_WALLS = #True
#TORCH_RADIUS = 10

;- -- Structure
Structure Object
  x.i
  y.i
  char.i
  color.i  
EndStructure

Structure Tile
  blocked.i
  block_sight.i
  explored.i
EndStructure

Structure rec
  x.i
  y.i
  w.i
  h.i
EndStructure

;- -- Variablen
Global con  ;Conselen Buffer

Global Dim TileMap.Tile(#MAP_WIDTH,#MAP_HEIGHT)

Global NewList Objects.Object() ;hold all Objects
Global *Player.Object = AddElement(Objects())

; -- FOV
Global fov_recompute = #True
Global fov_map

;- -- Colors
Global color_light_wall = RGB(230, 230, 230)
Global color_dark_wall = RGB(20, 20, 20)
Global color_light_ground = RGB(130,130,130)
Global color_dark_ground = RGB(35,35,35)

;- -- Declare Proceduren
Declare min(i1,i2)
Declare max(i1,i2)

;- -- Macro 
Macro RandomMinMax(Min,Max)
  Random(Max - Min) + Min
EndMacro
Macro SetTile(x,y,IsBlocked = #False)
  TileMap(x,y)\blocked = IsBlocked
  If IsBlocked
    TileMap(x,y)\block_sight = #True
  Else
    TileMap(x,y)\block_sight = #False
  EndIf
EndMacro
Macro SetRoom(Name,xx,yy,ww,hh)
  
  Name.rec\x = xx
  Name.rec\y = yy
  Name.rec\w = ww
  Name.rec\h = hh
EndMacro  
Macro RoomX2(room)
  room\x + room\w
EndMacro
Macro RoomY2(room)
  room\y + room\h
EndMacro
;- -- Proceduren
;- + Object
;move by the given amount
Procedure move(*o.Object, dx, dy)  
  If Not TileMap(*o\x + dx ,*o\y + dy)\blocked
    *o\x + dx
    *o\y + dy
  EndIf 
EndProcedure
;set the color and then draw the character that repesenzs zhis object at its position
Procedure draw(*o.Object)    
  TCOD_console_set_foreground_color(con, *o\color)
  TCOD_console_put_char(con, *o\x, *o\y, *o\char, #BKGND_NONE)
EndProcedure
;erase the character that represents this object
Procedure clear(*o.Object)  
  TCOD_console_put_char(con, *o\x, *o\y, Asc(" "), #BKGND_NONE)
EndProcedure
;- + Dungeon
Procedure create_room(*room.rec)
  For x = *room\x + 1  To *room\x + *room\w - 1  ;one for the Wall
    For y = *room\y + 1 To *room\y + *room\h - 1
      SetTile(x,y)      
    Next
  Next  
EndProcedure
Procedure create_h_tunnel(x1,x2,y)
  For x = min(x1,x2) To max(x1,x2)
    SetTile(x,y)
  Next  
EndProcedure
Procedure create_v_tunnel(y1,y2,x)
  For y = min(y1,y2) To max(y1,y2)
    SetTile(x,y)
  Next  
EndProcedure
Procedure room_center(*room.rec,*p.Point)
  *p\x = *room\x + (*room\w / 2)
  *p\y = *room\y + (*room\h / 2) 
EndProcedure
Procedure intersect(*room1.rec, *room2.rec)
  ProcedureReturn ((*room1\x <= RoomX2(*room2)) And (RoomX2(*room1) >= *room2\x) And (*room1\y <= RoomY2(*room2)) And (RoomY2(*room1) >= *room2\y))
EndProcedure
Procedure MakeMap()
   For x = 0 To ArraySize(TileMap(),1)
     For y = 0 To ArraySize(TileMap(),2)
       SetTile(x,y,#True)
     Next
   Next

  NewList rooms.rec()
  num_rooms = 0

  For r = 0 To #MAX_ROOMS    
    ;random width and height
    w = RandomMinMax(#ROOM_MINSIZE,#ROoM_MAX_SIZE)
    h = RandomMinMax(#ROOM_MINSIZE,#ROOM_MAX_SIZE)
    ;random position without going out of the boundaries of the map
    x = RandomMinMax(0,#MAP_WIDTH - w - 1)
    y = RandomMinMax(0,#MAP_HEIGHT - h -1)
    
    SetRoom(new_room,x,y,w,h)
    
    ;run through the other rooms and see if they intersect with this one
    failed = #False    
    ForEach rooms()      
      If intersect(@new_room,@rooms())
        failed = #True
        Break
      EndIf 
    Next
    
    If failed = #False
      ;this means there are no intersections, so this room is vailid
      
      ;paint it to the map       
      create_room(@new_room)
        
      ;center coordinates of new room, will be useful later        
      room_center(@new_room,@new_center.Point)
        
      If num_rooms = 0
        ;this is the first room, where the player starts at
        *Player\x = new_center\x
        *Player\y = new_center\y
      Else
        ;all rooms after the first:
        ;connect it to the previous room whith a tunnel
        
        ;center coordinates of previous romm
        room_center(rooms(),@prev_center.Point)
          
        ;draw a coin (random number that is either 0 or 1)
        If RandomMinMax(0,1) = 1
          ;first move horizontally, then vertically
          create_h_tunnel(prev_center\x,new_center\x,prev_center\y)
          create_v_tunnel(prev_center\y,new_center\y,new_center\x)
          ;create_v_tunnel(prev_center\y,new_center\y,prev_center\x)
        Else
          ;first move vertically, then horizontally
          create_v_tunnel(prev_center\y,new_center\y,prev_center\x)
          create_h_tunnel(prev_center\x,new_center\x,new_center\y)
          ;create_h_tunnel(prev_center\x,new_center\x,prev_center\y)
        EndIf                      
      EndIf
      
      ;finally, append the new room to the list
      AddElement(rooms())
      rooms() = new_room          
      num_rooms + 1
        
    EndIf
           
  Next r  
EndProcedure
;- + Common
Procedure Min(i1,i2)
  If i1 < i2
    ProcedureReturn i1 
  Else 
    ProcedureReturn i2
  EndIf 
EndProcedure    
Procedure Max(i1,i2)
  If i1 > i2
    ProcedureReturn i1 
  Else 
    ProcedureReturn i2
  EndIf 
EndProcedure
Procedure handle_keys() 
  key.TCOD_key_t
  TCOD_console_wait_for_keypress(@key,#True)
   
  ;alt+enter toggle fullscreen
  If key\vk = #KEY_ENTER And key\lalt
    TCOD_console_set_fullscreen(~(TCOD_console_is_fullscreen()))
  ElseIf key\vk = #KEY_ESCAPE
    ProcedureReturn #True    
  EndIf 
  
  ;movement keys
  If TCOD_console_is_key_pressed(#KEY_UP)
    move(*Player,0,-1)
    fov_recompute = #True
  ElseIf TCOD_console_is_key_pressed(#KEY_DOWN)
    move(*Player,0,1)
    fov_recompute = #True
  ElseIf TCOD_console_is_key_pressed(#KEY_LEFT)
    move(*Player,-1,0)
    fov_recompute = #True
  ElseIf TCOD_console_is_key_pressed(#KEY_RIGHT)
    move(*Player,1,0)
    fov_recompute = #True
  EndIf
  
EndProcedure
Procedure Render_all()
  
  If fov_recompute = #True
    ;recompute FOV if neede (the player moved or something)
    fov_recompute = #False
    TCOD_map_compute_fov(fov_map, *Player\x, *Player\y, #TORCH_RADIUS, #FOV_LIGHT_WALLS, #FOV_ALGO)
  EndIf   
  
  ForEach Objects()
    If TCOD_map_is_in_fov(fov_map, Objects()\x, Objects()\y)
      draw(@Objects())
    EndIf
  Next
   
    For x = 0 To ArraySize(TileMap(),1)
      For y = 0 To ArraySize(TileMap(),2)-1
       visible = TCOD_map_is_in_fov(fov_map, x, y) & $FF
       wall = TileMap(x,y)\block_sight
       
       If Not visible
         ;if it's not visible right now, the player can only see it if it's explored         
         If TileMap(x,y)\explored
           If wall
             TCOD_console_set_back(con,x,y,color_dark_wall,#BKGND_SET)
           Else
             TCOD_console_set_back(con,x,y,color_dark_ground,#BKGND_SET)
           EndIf         
         EndIf
       Else
         ;it's visible
         TileMap(x,y)\explored = #True
         If wall
           TCOD_console_set_back(con,x,y,color_light_wall,#BKGND_SET)
         Else
           TCOD_console_set_back(con,x,y,color_light_ground,#BKGND_SET)
         EndIf
       EndIf
       
       
;        If TileMap(x,y)\blocked = #True
;          TCOD_console_set_back(con,x,y,color_dark_wall,#BKGND_SET)
;        Else
;          TCOD_console_set_back(con,x,y,color_dark_ground,#BKGND_SET)
;        EndIf
     Next
   Next
   
   TCOD_console_blit(con, 0, 0, #SCREEN_WIDTH, #SCREEN_HEIGHT, 0, 0, 0)     
 EndProcedure
 
;- ++ Init ++

;- Files
If Not FileSize("arial10x10.png") > 0
  MessageRequester("Fehler", "arial10x10.png fehlt.")
  End
EndIf 
  
;- -- Tile Map
TileMap(30,22)\blocked = #True
TileMap(30,22)\block_sight = #True

;- -- Obejcts
; Player
*Player\x = 25
*Player\y = 23
*Player\char = Asc("@")
*Player\color = white
; NPC
AddElement(Objects())
Objects()\x = #SCREEN_WIDTH/2 - 5
Objects()\y = #SCREEN_HEIGHT/2
Objects()\char = Asc("@")
Objects()\color = yellow

;- -- Map
MakeMap()

;- -- Init Console, Load Font
TCOD_console_set_custom_font("arial10x10.png", #FONT_TYPE_GREYSCALE | #FONT_LAYOUT_TCOD,0,0)
TCOD_console_init_root(#SCREEN_WIDTH, #SCREEN_HEIGHT, "libtcod PB Wraper", #False)
con = TCOD_console_new(#SCREEN_WIDTH, #SCREEN_HEIGHT)

;- -- Field of View
fov_map = TCOD_map_new(#MAP_WIDTH, #MAP_HEIGHT)
For y = 0 To #MAP_HEIGHT
  For x = 0 To #MAP_WIDTH
    TCOD_map_set_properties(fov_map, x, y, ~TileMap(x,y)\block_sight, ~TileMap(x,y)\blocked)
  Next x
Next y

;- ++ Main Loop ++
While TCOD_console_is_window_closed()&$FF = #False
  
   ;Draw all Objects
   Render_all()
   
   TCOD_console_flush()
   
   ;erase Objects
   ForEach Objects()
     clear(@Objects())
   Next
   
   ;handle keys And exit If needed
   exit = handle_keys()
   If exit : Break  : EndIf 
   
 Wend
 
 CloseLibrary(libtcod)
 
 End
 ; ________________________________________________________________________________
 ;- ENDE
 ; ________________________________________________________________________________

; IDE Options = PureBasic 4.60 (Windows - x86)
; CursorPosition = 313
; FirstLine = 84
; Folding = AAA-
; EnableXP